package lightcontainer.storage;

import com.google.gson.JsonArray;
import com.google.gson.JsonObject;

import java.util.Iterator;

public class JsonAdapter implements Adapter {

    @Override
    public String toString(AppData appData) {
        return addData(appData);
    }

    private String addData(AppData appData) {
        AppConfig appConfig = appData.getAppConfig();
        JsonObject config = new JsonObject();
        config.addProperty("unicast_port", appConfig.getUnicastPort());
        config.addProperty("multicast_ip", appConfig.getMulticastIp());
        config.addProperty("multicast_port", appConfig.getMulticastPort());
        config.addProperty("network_interface", appConfig.getNetworkInterface());
        config.addProperty("tls", appConfig.isTls());
        JsonArray users = new JsonArray();
        Iterator<User> userIterator = appData.usersIterator();
        addUsers(users, userIterator);
        config.addProperty("users", users.toString());
        return config.toString();
    }

    private void addUsers(JsonArray users, Iterator<User> userIterator) {
        while (userIterator.hasNext()) {
            User current = userIterator.next();
            JsonObject user = new JsonObject();
            user.addProperty("name", current.getName());
            user.addProperty("password", current.getPassword());
            user.addProperty("aes_key", current.getAesKey());
            JsonArray files = new JsonArray();
            Iterator<File> fileIterator = current.fileIterator();
            addFiles(fileIterator);
            user.addProperty("files", files.toString());
            users.add(user);
        }
    }

    private void addFiles(Iterator<File> fileIterator) {
        while (fileIterator.hasNext()) {
            File currentFile = fileIterator.next();
            JsonObject file = new JsonObject();
            file.addProperty("name", currentFile.getName());
            file.addProperty("size", currentFile.getSize());
            file.addProperty("iv", currentFile.getIv());
            JsonArray storage = new JsonArray();
            Iterator<String> storageIterator = currentFile.getStorageIterator();
            addStorage(storage, storageIterator);
            file.addProperty("storage", storage.toString());
        }
    }

    private void addStorage(JsonArray storage, Iterator<String> storageIterator) {
        while (storageIterator.hasNext()) {
            String storageString = storageIterator.next();
            storage.add(storageString);
        }
    }

    @Override
    public AppData fromString(String appDataString) {
        return null;
    }
}
